@@ -1,11 +1,11 @@
 /*
-  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
-  You may not use this file except in compliance with the License.
+  You may not use this file except in compliance with the License.  You may
   obtain a copy of the License at
   
-    http://www.imagemagick.org/script/license.php
+    https://imagemagick.org/script/license.php
   
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,24 +15,22 @@
 
   MagickCore Windows NT utility methods.
 */
-#ifndef _MAGICKCORE_NT_FEATURE_H
-#define _MAGICKCORE_NT_FEATURE_H
+#ifndef MAGICKCORE_NT_FEATURE_H
+#define MAGICKCORE_NT_FEATURE_H
+
+#include "magick/exception.h"
+#include "magick/geometry.h"
+#include "magick/splay-tree.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#include "magick/splay-tree.h"
-
-extern MagickExport void
-  *CropImageToHBITMAP(Image *,const RectangleInfo *,ExceptionInfo *),
-  *ImageToHBITMAP(Image *);
-
 #if !defined(XS_VERSION)
 
 extern MagickExport MagickBooleanType
-  NTIsMagickConflict(const char *),
-  NTLoadTypeLists(SplayTreeInfo *,ExceptionInfo *);
+  NTAcquireTypeCache(SplayTreeInfo *,ExceptionInfo *),
+  NTIsMagickConflict(const char *);
 
 #endif
 
